module App.Page exposing
    ( Page
    , Transition(..)
    , componentCatalogPanel
    , content
    , drawer
    , drawerItems
    , subheader
    , topappbar
    )

import App.Url as Url exposing (Url)
import Array exposing (Array)
import Html exposing (Html, div, h2, section, span, text)
import Html.Attributes as Html
import Material
import Material.Drawer.Dismissible as DismissibleDrawer
import Material.Drawer.Modal as ModalDrawer
import Material.List as Lists
import Material.Options as Options exposing (Property, cs, css, styled, when)
import Material.TopAppBar as TopAppBar
import Material.Typography as Typography


type alias Page m =
    { topappbar : String -> Html m
    , navigate : Url -> m
    , body : List (Html m) -> Html m
    }


topappbar :
    (Material.Msg m -> m)
    -> Material.Index
    -> Material.Model m
    -> m
    -> Url
    -> String
    -> Html m
topappbar lift idx mdc cmd url title =
    TopAppBar.view lift
        idx
        mdc
        [ cs "catalog-top-app-bar"
        ]
        [ TopAppBar.section
            [ TopAppBar.alignStart
            ]
            [ case url of
                Url.StartPage ->
                    styled Html.img
                        [ cs "mdc-toolbar__menu-icon"
                        , Options.attribute (Html.src "images/logo_48.jpeg")
                        ]
                        []

                _ ->
                    TopAppBar.navigationIcon lift
                        "my-menu"
                        mdc
                        [ Options.onClick cmd ]
                        "menu"
            , TopAppBar.title
                [ cs "catalog-top-app-bar__title"
                ]
                [ styled span [ cs "catalog-top-app-bar__title--small-screen" ] [ text title ]
                , styled span [ cs "catalog-top-app-bar__title--large-screen" ] [ text title ]
                ]
            ]
        ]


drawerItems : Array ( String, Url )
drawerItems =
    Array.fromList
        [ ( "Home", Url.StartPage )

        --        , ( "URL1", Url.Url1 )
        --        , ( "URL2", Url.Url2 )
        --        , ( "URL3", Url.Url3 )
        ]


drawer :
    (Material.Msg m -> m)
    -> Material.Index
    -> Material.Model m
    -> m
    -> (Int -> m)
    -> Url
    -> Bool
    -> Bool
    -> Html m
drawer lift idx mdc close select current_url dismissible_drawer open =
    let
        a ( title, url ) =
            listItem title url current_url
    in
    (if dismissible_drawer then
        DismissibleDrawer.view

     else
        ModalDrawer.view
    )
        lift
        idx
        mdc
        [ ModalDrawer.open |> when open
        , ModalDrawer.onClose close
        , TopAppBar.fixedAdjust
        , css "z-index" "1" |> when dismissible_drawer
        ]
        [ ModalDrawer.header
            [ css "position" "absolute"
            , css "top" "18px"
            , css "opacity" ".74"
            , css "padding" "0 16px 4px 16px"
            ]
            [ styled Html.img
                [ Options.attribute (Html.src "https://material-components.github.io/material-components-web-catalog/static/media/ic_material_design_24px.svg")
                ]
                []
            ]
        , ModalDrawer.content
            []
            [ Lists.nav lift
                "drawer-list"
                mdc
                [ Lists.singleSelection
                , Lists.useActivated
                , Lists.onSelectListItem select
                ]
                (List.map a (Array.toList drawerItems))
            ]
        ]


listItem : String -> Url -> Url -> Lists.ListItem m
listItem title url current_url =
    Lists.a
        [ Lists.activated |> when (current_url == url)
        ]
        [ text title
        ]


subheader : String -> Html m
subheader title =
    styled Html.h3
        [ Typography.subtitle1 ]
        [ text title ]


type Transition
    = None
    | Enter
    | Active
    | Done


componentCatalogPanel : Transition -> List (Html m) -> Html m
componentCatalogPanel transition nodes =
    styled section
        [ cs "component-catalog-panel"
        , cs "loadComponent-enter" |> when (transition == Enter || transition == Active)
        , cs "loadComponent-enter-active" |> when (transition == Active)
        , css "margin-top" "24px"
        , css "padding-bottom" "24px"
        ]
        nodes


content : String -> List (Html m) -> Html m
content name nodes =
    styled div
        [ css "padding-bottom" "20px" ]
        (styled h2
            [ Typography.headline6
            , css "border-bottom" "1px solid rgba(0,0,0,.87)"
            ]
            [ text name ]
            :: nodes
        )
